"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var rxjs_1 = require("rxjs");
var terminate_1 = require("../ssr/terminate");
var redux_devtools_extension_1 = require("../redux-devtools-extension");
function createState(reducer, effect) {
    var action$ = new rxjs_1.Subject();
    var _action$ = new rxjs_1.Subject();
    var stateObservers = new Set();
    var actionObservers = new Set();
    var state$ = new rxjs_1.ReplaySubject(1);
    function stateCreator(defaultState, middleware, loadFromSSR) {
        if (middleware === void 0) { middleware = rxjs_1.identity; }
        if (loadFromSSR === void 0) { loadFromSSR = false; }
        var state = Object.create(null);
        var appState = defaultState;
        function dispatch(action) {
            if (action.state !== state && action.type !== terminate_1.TERMINATE_ACTION.type) {
                action.state.dispatch(action);
                return;
            }
            var prevState = appState;
            var newState = reducer(prevState, action);
            if (newState !== prevState) {
                state$.next(newState);
            }
            redux_devtools_extension_1.logStateAction(action);
            action$.next(action);
            _action$.next(action);
        }
        var effect$ = effect(_action$, loadFromSSR);
        var subscription = new rxjs_1.Subscription();
        subscription.add(middleware(effect$).subscribe(function (action) {
            try {
                dispatch(action);
            }
            catch (e) {
                action$.error(e);
            }
        }, function (err) {
            console.error(err);
        }));
        subscription.add(action$.subscribe(function (action) {
            var e_1, _a;
            try {
                for (var actionObservers_1 = tslib_1.__values(actionObservers), actionObservers_1_1 = actionObservers_1.next(); !actionObservers_1_1.done; actionObservers_1_1 = actionObservers_1.next()) {
                    var observer = actionObservers_1_1.value;
                    observer(action);
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (actionObservers_1_1 && !actionObservers_1_1.done && (_a = actionObservers_1.return)) _a.call(actionObservers_1);
                }
                finally { if (e_1) throw e_1.error; }
            }
        }, function (err) {
            _action$.error(err);
        }, function () {
            _action$.complete();
        }));
        subscription.add(state$.subscribe(function (state) {
            var e_2, _a;
            appState = state;
            try {
                for (var stateObservers_1 = tslib_1.__values(stateObservers), stateObservers_1_1 = stateObservers_1.next(); !stateObservers_1_1.done; stateObservers_1_1 = stateObservers_1.next()) {
                    var observer = stateObservers_1_1.value;
                    observer(state);
                }
            }
            catch (e_2_1) { e_2 = { error: e_2_1 }; }
            finally {
                try {
                    if (stateObservers_1_1 && !stateObservers_1_1.done && (_a = stateObservers_1.return)) _a.call(stateObservers_1);
                }
                finally { if (e_2) throw e_2.error; }
            }
        }));
        state$.next(defaultState);
        Object.assign(state, {
            dispatch: dispatch,
            state$: state$,
            getState: function () { return appState; },
            subscribeState: function (observer) {
                stateObservers.add(observer);
                return function () { return stateObservers.delete(observer); };
            },
            subscribeAction: function (observer) {
                actionObservers.add(observer);
                return function () { return actionObservers.delete(observer); };
            },
            unsubscribe: function () {
                action$.complete();
                state$.complete();
                subscription.unsubscribe();
                state.dispatch = rxjs_1.noop;
                stateObservers.clear();
                actionObservers.clear();
            },
        });
        return state;
    }
    return { stateCreator: stateCreator, action$: action$, state$: state$ };
}
exports.createState = createState;
